@@ -6,4 +6,4 @@ usermod -u $LOCAL_USER_ID $USERNAME
 usermod -g $LOCAL_GROUP_ID $USERNAME
 export HOME=/home/$USERNAME
 
-exec gosu $USERNAME "/opt/cutter/build/Cutter" $@
+exec su-exec $USERNAME "/opt/cutter/build/Cutter" $@
