@@ -4,8 +4,9 @@
 #include <QEvent>
 #include <QApplication>
 #include <QList>
+#include <QProxyStyle>
 
-#include "MainWindow.h"
+#include "core/MainWindow.h"
 
 
 class CutterApplication : public QApplication
@@ -21,15 +22,13 @@ public:
         return mainWindow;
     }
 
-    void loadPlugins();
-
 protected:
     bool event(QEvent *e);
 
 private:
-    /*!
-     * \brief Load and translations depending on Language settings
-     * \return true on success
+    /**
+     * @brief Load and translations depending on Language settings
+     * @return true on success
      */
     bool loadTranslations();
 
@@ -38,4 +37,18 @@ private:
     MainWindow *mainWindow;
 };
 
+
+/**
+ * @brief CutterProxyStyle is used to force shortcuts displaying in context menu
+ */
+class CutterProxyStyle : public QProxyStyle
+{
+    Q_OBJECT
+public:
+    /**
+     * @brief it is enough to get notification about QMenu polishing to force shortcut displaying
+     */
+    void polish(QWidget *widget) override;
+};
+
 #endif // CUTTERAPPLICATION_H
