@@ -1,4 +1,4 @@
-#include "MainWindow.h"
+#include "core/MainWindow.h"
 #include "CutterSeekable.h"
 
 #include <QPlainTextEdit>
@@ -13,6 +13,13 @@ CutterSeekable::CutterSeekable(QObject *parent)
 
 CutterSeekable::~CutterSeekable() {}
 
+void CutterSeekable::setSynchronization(bool sync)
+{
+    synchronized = sync;
+    onCoreSeekChanged(Core()->getOffset());
+    emit syncChanged();
+}
+
 void CutterSeekable::onCoreSeekChanged(RVA addr)
 {
     if (synchronized && widgetOffset != addr) {
@@ -48,11 +55,32 @@ RVA CutterSeekable::getOffset()
 
 void CutterSeekable::toggleSynchronization()
 {
-    synchronized = !synchronized;
-    onCoreSeekChanged(Core()->getOffset());
+    setSynchronization(!synchronized);
 }
 
 bool CutterSeekable::isSynchronized()
 {
     return synchronized;
 }
+
+void CutterSeekable::seekToReference(RVA offset)
+{
+    if (offset == RVA_INVALID)
+    {
+        return;
+    }
+    
+    RVA target;
+    QList<XrefDescription> refs = Core()->getXRefs(offset, false, false);
+    
+    if (refs.length()) {
+        if (refs.length() > 1) {
+            qWarning() << "Too many references here. Weird behaviour expected.";
+        }
+        
+        target = refs.at(0).to;
+        if (target != RVA_INVALID) {
+            seek(target);
+        }
+    }
+}
\ No newline at end of file
