@@ -3,6 +3,7 @@
 #define SVGICONENGINE_H
 
 #include <QIconEngine>
+#include <QPalette>
 
 class SvgIconEngine: public QIconEngine
 {
@@ -11,10 +12,9 @@ private:
 
 public:
     explicit SvgIconEngine(const QString &filename);
-    explicit SvgIconEngine(const QByteArray &svgData);
 
-    SvgIconEngine(const QString &filename, QColor tintColor);
-    SvgIconEngine(const QByteArray &svgData, QColor tintColor);
+    SvgIconEngine(const QString &filename, const QColor &tintColor);
+    SvgIconEngine(const QString &filename, QPalette::ColorRole colorRole);
 
     void paint(QPainter *painter, const QRect &rect, QIcon::Mode mode, QIcon::State state) override;
     QIconEngine *clone() const override;
