@@ -1,4 +1,5 @@
-#pragma once
+#ifndef SYNTAXHIGHLIGHTER_H
+#define SYNTAXHIGHLIGHTER_H
 
 #include <QSyntaxHighlighter>
 #include <QVector>
@@ -6,14 +7,33 @@
 #include <QRegularExpression>
 #include <QTextCharFormat>
 
+#ifdef CUTTER_ENABLE_KSYNTAXHIGHLIGHTING
 
-class SyntaxHighlighter : public QSyntaxHighlighter
+#include <KSyntaxHighlighting/syntaxhighlighter.h>
+
+class SyntaxHighlighter : public KSyntaxHighlighting::SyntaxHighlighter
 {
     Q_OBJECT
 
 public:
-    SyntaxHighlighter(QTextDocument *parent = nullptr);
-    virtual ~SyntaxHighlighter() = default;
+    SyntaxHighlighter(QTextDocument *document);
+
+private slots:
+    void updateTheme();
+};
+
+#endif
+
+/**
+ * SyntaxHighlighter to be used when KSyntaxHighlighting is not available
+ */
+class FallbackSyntaxHighlighter : public QSyntaxHighlighter
+{
+    Q_OBJECT
+
+public:
+    FallbackSyntaxHighlighter(QTextDocument *parent = nullptr);
+    virtual ~FallbackSyntaxHighlighter() = default;
 
 protected:
     void highlightBlock(const QString &text) override;
@@ -31,3 +51,5 @@ private:
 
     QTextCharFormat multiLineCommentFormat;
 };
+
+#endif
