@@ -1,12 +1,12 @@
 
 #include <cassert>
 
-#include "Cutter.h"
+#include "core/Cutter.h"
 #include "TempConfig.h"
 
 TempConfig::~TempConfig()
 {
-    for (auto i = resetValues.constBegin(); i != resetValues.constEnd(); i++) {
+    for (auto i = resetValues.constBegin(); i != resetValues.constEnd(); ++i) {
         switch (i.value().type()) {
         case QVariant::String:
             Core()->setConfig(i.key(), i.value().toString());
