@@ -1,6 +1,8 @@
 #include "CommentsDialog.h"
 #include "ui_CommentsDialog.h"
 
+#include "core/Cutter.h"
+
 CommentsDialog::CommentsDialog(QWidget *parent) :
     QDialog(parent),
     ui(new Ui::CommentsDialog)
@@ -34,6 +36,30 @@ void CommentsDialog::setComment(const QString &comment)
     ui->textEdit->document()->setPlainText(comment);
 }
 
+void CommentsDialog::addOrEditComment(RVA offset, QWidget *parent)
+{
+    QString oldComment = Core()->cmd("CC." + RAddressString(offset));
+    // Remove newline at the end added by cmd
+    oldComment.remove(oldComment.length() - 1, 1);
+    CommentsDialog c(parent);
+
+    if (oldComment.isNull() || oldComment.isEmpty()) {
+        c.setWindowTitle(tr("Add Comment at %1").arg(RAddressString(offset)));
+    } else {
+        c.setWindowTitle(tr("Edit Comment at %1").arg(RAddressString(offset)));
+    }
+
+    c.setComment(oldComment);
+    if (c.exec()) {
+        QString comment = c.getComment();
+        if (comment.isEmpty()) {
+            Core()->delComment(offset);
+        } else {
+            Core()->setComment(offset, comment);
+        }
+    }
+}
+
 bool CommentsDialog::eventFilter(QObject */*obj*/, QEvent *event)
 {
     if (event -> type() == QEvent::KeyPress) {
