@@ -33,17 +33,6 @@ void EditFunctionDialog::setStartAddrText(const QString &startAddr)
     ui->startLineEdit->setText(startAddr);
 }
 
-QString EditFunctionDialog::getEndAddrText()
-{
-    QString ret = ui->endLineEdit->text();
-    return ret;
-}
-
-void EditFunctionDialog::setEndAddrText(const QString &endAddr)
-{
-    ui->endLineEdit->setText(endAddr);
-}
-
 QString EditFunctionDialog::getStackSizeText()
 {
     QString ret = ui->stackSizeLineEdit->text();
@@ -55,11 +44,11 @@ void EditFunctionDialog::setStackSizeText(const QString &stackSize)
     ui->stackSizeLineEdit->setText(stackSize);
 }
 
-void EditFunctionDialog::setCallConList(const QStringList callConList) {
+void EditFunctionDialog::setCallConList(const QStringList &callConList) {
     ui->callConComboBox->addItems(callConList);
 }
 
-void EditFunctionDialog::setCallConSelected(const QString selected) {
+void EditFunctionDialog::setCallConSelected(const QString &selected) {
     ui->callConComboBox->setCurrentText(selected);
 }
 
