@@ -1,11 +1,14 @@
 #include "FlagDialog.h"
 #include "ui_FlagDialog.h"
 
+#include <QIntValidator>
+#include "core/Cutter.h"
+
+
 FlagDialog::FlagDialog(RVA offset, QWidget *parent) :
     QDialog(parent),
     ui(new Ui::FlagDialog),
-    offset(offset),
-    core(Core())
+    offset(offset)
 {
     ui->setupUi(this);
     setWindowFlags(windowFlags() & (~Qt::WindowContextHelpButtonHint));
@@ -21,7 +24,7 @@ void FlagDialog::on_buttonBox_accepted()
 {
     QString name = ui->nameEdit->text();
     RVA size = ui->sizeEdit->text().toULongLong();
-    core->addFlag(offset, name, size);
+    Core()->addFlag(offset, name, size);
 }
 
 void FlagDialog::on_buttonBox_rejected()
