@@ -3,7 +3,7 @@
 
 #include <QDialog>
 #include <memory>
-#include "Cutter.h"
+#include "core/Cutter.h"
 
 namespace Ui {
 class OpenFileDialog;
