@@ -6,7 +6,7 @@
 #include <QPushButton>
 #include <memory>
 
-#include "Cutter.h"
+#include "core/Cutter.h"
 
 class PreferencesDialog;
 
@@ -19,11 +19,16 @@ class AsmOptionsWidget : public QDialog
     Q_OBJECT
 
 public:
-    explicit AsmOptionsWidget(PreferencesDialog *dialog, QWidget *parent = nullptr);
+    explicit AsmOptionsWidget(PreferencesDialog *dialog);
     ~AsmOptionsWidget();
 
 private:
     std::unique_ptr<Ui::AsmOptionsWidget> ui;
+    struct ConfigCheckbox {
+        QCheckBox *checkBox;
+        QString config;
+    };
+    QList<ConfigCheckbox> checkboxes;
 
     void triggerAsmOptionsChanged();
 
@@ -32,33 +37,23 @@ private slots:
 
     void updateAsmOptionsFromVars();
 
-    void on_esilCheckBox_toggled(bool checked);
-    void on_pseudoCheckBox_toggled(bool checked);
-    void on_offsetCheckBox_toggled(bool checked);
-    void on_describeCheckBox_toggled(bool checked);
-    void on_stackpointerCheckBox_toggled(bool checked);
-    void on_slowCheckBox_toggled(bool checked);
-    void on_linesCheckBox_toggled(bool checked);
-    void on_fcnlinesCheckBox_toggled(bool checked);
-    void on_flgoffCheckBox_toggled(bool checked);
-    void on_emuCheckBox_toggled(bool checked);
-    void on_cmtrightCheckBox_toggled(bool checked);
+
     void on_cmtcolSpinBox_valueChanged(int value);
-    void on_varsumCheckBox_toggled(bool checked);
-    void on_bytesCheckBox_toggled(bool checked);
-    void on_sizeCheckBox_toggled(bool checked);
-    void on_bytespaceCheckBox_toggled(bool checked);
-    void on_lbytesCheckBox_toggled(bool checked);
+
     void on_syntaxComboBox_currentIndexChanged(int index);
     void on_caseComboBox_currentIndexChanged(int index);
     void on_asmTabsSpinBox_valueChanged(int value);
     void on_asmTabsOffSpinBox_valueChanged(int value);
     void on_nbytesSpinBox_valueChanged(int value);
-    void on_bblineCheckBox_toggled(bool checked);
+
+    void on_bytesCheckBox_toggled(bool checked);
     void on_varsubCheckBox_toggled(bool checked);
-    void on_varsubOnlyCheckBox_toggled(bool checked);
 
     void on_buttonBox_clicked(QAbstractButton *button);
+
+    void commentsComboBoxChanged(int index);
+    void asmComboBoxChanged(int index);
+    void checkboxEnabler(QCheckBox *checkbox, QString config);
 };
 
 
