@@ -3,7 +3,7 @@
 
 #include <memory>
 
-#include "Cutter.h"
+#include "core/Cutter.h"
 
 class PreferencesDialog;
 
@@ -16,7 +16,7 @@ class DebugOptionsWidget : public QDialog
     Q_OBJECT
 
 public:
-    explicit DebugOptionsWidget(PreferencesDialog *dialog, QWidget *parent = nullptr);
+    explicit DebugOptionsWidget(PreferencesDialog *dialog);
     ~DebugOptionsWidget();
 
 private:
@@ -24,7 +24,6 @@ private:
 
 private slots:
     void updateDebugPlugin();
-    void updateDebugArgs();
     void updateStackAddr();
     void updateStackSize();
     void on_pluginComboBox_currentIndexChanged(const QString &index);
