@@ -9,12 +9,10 @@
 #include "common/Helpers.h"
 #include "common/Configuration.h"
 
-GraphOptionsWidget::GraphOptionsWidget(PreferencesDialog *dialog, QWidget *parent)
-    : QDialog(parent),
+GraphOptionsWidget::GraphOptionsWidget(PreferencesDialog *dialog)
+    : QDialog(dialog),
       ui(new Ui::GraphOptionsWidget)
 {
-    Q_UNUSED(dialog)
-
     ui->setupUi(this);
 
     updateOptionsFromVars();
