@@ -8,25 +8,80 @@
 #include "dialogs/EditVariablesDialog.h"
 #include "dialogs/SetToDataDialog.h"
 #include "dialogs/EditFunctionDialog.h"
+#include "dialogs/LinkTypeDialog.h"
+#include "dialogs/EditStringDialog.h"
+#include "dialogs/BreakpointsDialog.h"
+#include "MainWindow.h"
+
 #include <QtCore>
 #include <QShortcut>
 #include <QJsonArray>
 #include <QClipboard>
 #include <QApplication>
+#include <QPushButton>
 
-DisassemblyContextMenu::DisassemblyContextMenu(QWidget *parent)
+DisassemblyContextMenu::DisassemblyContextMenu(QWidget *parent, MainWindow *mainWindow)
     :   QMenu(parent),
         offset(0),
-        canCopy(false)
+        canCopy(false),
+        mainWindow(mainWindow),
+        actionEditInstruction(this),
+        actionNopInstruction(this),
+        actionJmpReverse(this),
+        actionEditBytes(this),
+        actionCopy(this),
+        actionCopyAddr(this),
+        actionAddComment(this),
+        actionAddFlag(this),
+        actionAnalyzeFunction(this),
+        actionEditFunction(this),
+        actionRename(this),
+        actionRenameUsedHere(this),
+        actionSetFunctionVarTypes(this),
+        actionXRefs(this),
+        actionDisplayOptions(this),
+        actionDeleteComment(this),
+        actionDeleteFlag(this),
+        actionDeleteFunction(this),
+        actionLinkType(this),
+        actionSetBaseBinary(this),
+        actionSetBaseOctal(this),
+        actionSetBaseDecimal(this),
+        actionSetBaseHexadecimal(this),
+        actionSetBasePort(this),
+        actionSetBaseIPAddr(this),
+        actionSetBaseSyscall(this),
+        actionSetBaseString(this),
+        actionSetBits16(this),
+        actionSetBits32(this),
+        actionSetBits64(this),
+        actionContinueUntil(this),
+        actionAddBreakpoint(this),
+        actionAdvancedBreakpoint(this),
+        actionSetPC(this),
+        actionSetToCode(this),
+        actionSetAsStringAuto(this),
+        actionSetAsStringRemove(this),
+        actionSetAsStringAdvanced(this),
+        actionSetToDataEx(this),
+        actionSetToDataByte(this),
+        actionSetToDataWord(this),
+        actionSetToDataDword(this),
+        actionSetToDataQword(this),
+        showInSubmenu(this)
 {
     initAction(&actionCopy, tr("Copy"), SLOT(on_actionCopy_triggered()), getCopySequence());
     addAction(&actionCopy);
 
-    copySeparator = addSeparator();
-
-    initAction(&actionCopyAddr, tr("Copy address"), SLOT(on_actionCopyAddr_triggered()));
+    initAction(&actionCopyAddr, tr("Copy address"), SLOT(on_actionCopyAddr_triggered()),
+               getCopyAddressSequence());
     addAction(&actionCopyAddr);
 
+    initAction(&showInSubmenu, tr("Show in"), nullptr);
+    addAction(&showInSubmenu);
+
+    copySeparator = addSeparator();
+
     initAction(&actionAddComment, tr("Add Comment"),
                SLOT(on_actionAddComment_triggered()), getCommentSequence());
     addAction(&actionAddComment);
@@ -40,7 +95,7 @@ DisassemblyContextMenu::DisassemblyContextMenu(QWidget *parent)
     addAction(&actionRename);
 
     initAction(&actionEditFunction, tr("Edit function"),
-               SLOT(on_actionEditFunction_triggered()));
+               SLOT(on_actionEditFunction_triggered()), getEditFunctionSequence());
     addAction(&actionEditFunction);
 
     initAction(&actionRenameUsedHere, tr("Rename Flag/Fcn/Var Used Here"),
@@ -58,11 +113,11 @@ DisassemblyContextMenu::DisassemblyContextMenu(QWidget *parent)
     addAction(&actionDeleteFlag);
 
     initAction(&actionDeleteFunction, tr("Undefine function"),
-               SLOT(on_actionDeleteFunction_triggered()));
+               SLOT(on_actionDeleteFunction_triggered()), getUndefineFunctionSequence());
     addAction(&actionDeleteFunction);
 
-    initAction(&actionAnalyzeFunction, tr("Define function here..."),
-               SLOT(on_actionAnalyzeFunction_triggered()));
+    initAction(&actionAnalyzeFunction, tr("Define function here"),
+               SLOT(on_actionAnalyzeFunction_triggered()), getDefineNewFunctionSequence());
     addAction(&actionAnalyzeFunction);
 
     addSeparator();
@@ -71,15 +126,15 @@ DisassemblyContextMenu::DisassemblyContextMenu(QWidget *parent)
 
     addSetBitsMenu();
 
-    initAction(&actionSetToCode, tr("Set as Code"),
-               SLOT(on_actionSetToCode_triggered()), getSetToCodeSequence());
-    addAction(&actionSetToCode);
+    structureOffsetMenu = addMenu(tr("Structure offset"));
+    connect(structureOffsetMenu, SIGNAL(triggered(QAction*)),
+            this, SLOT(on_actionStructureOffsetMenu_triggered(QAction*)));
 
-    initAction(&actionSetAsString, tr("Set as String"),
-               SLOT(on_actionSetAsString_triggered()), getSetAsStringSequence());
-    addAction(&actionSetAsString);
+    initAction(&actionLinkType, tr("Link Type to Address"),
+               SLOT(on_actionLinkType_triggered()), getLinkTypeSequence());
+    addAction(&actionLinkType);
 
-    addSetToDataMenu();
+    addSetAsMenu();
 
     addSeparator();
 
@@ -96,17 +151,22 @@ DisassemblyContextMenu::DisassemblyContextMenu(QWidget *parent)
 
     addSeparator();
 
+    addBreakpointMenu();
     addDebugMenu();
 
+    addSeparator();
+
+    pluginMenu = mainWindow->getContextMenuExtensions(MainWindow::ContextMenuType::Disassembly);
+    pluginActionMenuAction = addMenu(pluginMenu);
+
+    addSeparator();
+
     connect(this, &DisassemblyContextMenu::aboutToShow,
             this, &DisassemblyContextMenu::aboutToShowSlot);
 }
 
 DisassemblyContextMenu::~DisassemblyContextMenu()
 {
-    for (QAction *action : anonymousActions) {
-        delete action;
-    }
 }
 
 void DisassemblyContextMenu::addSetBaseMenu()
@@ -163,9 +223,35 @@ void DisassemblyContextMenu::addSetBitsMenu()
     connect(&actionSetBits64, &QAction::triggered, this, [this] { setBits(64); });
 }
 
+
+void DisassemblyContextMenu::addSetAsMenu()
+{
+    setAsMenu = addMenu(tr("Set as..."));
+
+    initAction(&actionSetToCode, tr("Code"),
+               SLOT(on_actionSetToCode_triggered()), getSetToCodeSequence());
+    setAsMenu->addAction(&actionSetToCode);
+
+    setAsString = setAsMenu->addMenu(tr("String..."));
+
+    initAction(&actionSetAsStringAuto, tr("Auto-detect"),
+               SLOT(on_actionSetAsString_triggered()), getSetAsStringSequence());
+    initAction(&actionSetAsStringRemove, tr("Remove"),
+               SLOT(on_actionSetAsStringRemove_triggered()));
+    initAction(&actionSetAsStringAdvanced, tr("Adanced"),
+               SLOT(on_actionSetAsStringAdvanced_triggered()));
+
+
+    setAsString->addAction(&actionSetAsStringAuto);
+    setAsString->addAction(&actionSetAsStringRemove);
+    setAsString->addAction(&actionSetAsStringAdvanced);
+
+    addSetToDataMenu();
+}
+
 void DisassemblyContextMenu::addSetToDataMenu()
 {
-    setToDataMenu = addMenu(tr("Set to Data..."));
+    setToDataMenu = setAsMenu->addMenu(tr("Data..."));
 
     initAction(&actionSetToDataByte, tr("Byte"));
     setToDataMenu->addAction(&actionSetToDataByte);
@@ -187,7 +273,7 @@ void DisassemblyContextMenu::addSetToDataMenu()
                SLOT(on_actionSetToDataEx_triggered()), getSetToDataExSequence());
     setToDataMenu->addAction(&actionSetToDataEx);
 
-    auto switchAction = new QAction();
+    auto switchAction = new QAction(this);
     initAction(switchAction, "Switch Data",
                SLOT(on_actionSetToData_triggered()), getSetToDataSequence());
 }
@@ -209,13 +295,21 @@ void DisassemblyContextMenu::addEditMenu()
     editMenu->addAction(&actionJmpReverse);
 }
 
-void DisassemblyContextMenu::addDebugMenu()
+void DisassemblyContextMenu::addBreakpointMenu()
 {
-    debugMenu = addMenu(tr("Debug"));
+    breakpointMenu = addMenu(tr("Breakpoint"));
 
     initAction(&actionAddBreakpoint, tr("Add/remove breakpoint"),
                SLOT(on_actionAddBreakpoint_triggered()), getAddBPSequence());
-    debugMenu->addAction(&actionAddBreakpoint);
+    breakpointMenu->addAction(&actionAddBreakpoint);
+    initAction(&actionAdvancedBreakpoint, tr("Advanced breakpoint"),
+               SLOT(on_actionAdvancedBreakpoint_triggered()), QKeySequence(Qt::CTRL+Qt::Key_F2));
+    breakpointMenu->addAction(&actionAdvancedBreakpoint);
+}
+
+void DisassemblyContextMenu::addDebugMenu()
+{
+    debugMenu = addMenu(tr("Debug"));
 
     initAction(&actionContinueUntil, tr("Continue until line"),
                SLOT(on_actionContinueUntil_triggered()));
@@ -225,9 +319,70 @@ void DisassemblyContextMenu::addDebugMenu()
     debugMenu->addAction(&actionSetPC);
 }
 
+QVector<DisassemblyContextMenu::ThingUsedHere> DisassemblyContextMenu::getThingUsedHere(RVA offset)
+{
+    QVector<ThingUsedHere> result;
+    const QJsonArray array = Core()->cmdj("anj @ " + QString::number(offset)).array();
+    result.reserve(array.size());
+    for (const auto &thing : array) {
+        auto obj = thing.toObject();
+        RVA offset = obj["offset"].toVariant().toULongLong();
+        QString name = obj["name"].toString();
+        QString typeString = obj["type"].toString();
+        ThingUsedHere::Type type = ThingUsedHere::Type::Address;
+        if (typeString == "var") {
+            type = ThingUsedHere::Type::Var;
+        } else if (typeString == "flag") {
+            type = ThingUsedHere::Type::Flag;
+        } else if (typeString == "function") {
+            type = ThingUsedHere::Type::Function;
+        } else if (typeString == "address") {
+            type = ThingUsedHere::Type::Address;
+        }
+        result.push_back(ThingUsedHere{name, offset, type});
+    }
+    return result;
+}
+
+void DisassemblyContextMenu::updateTargetMenuActions(const QVector<ThingUsedHere> &targets)
+{
+    for (auto action : showTargetMenuActions) {
+        removeAction(action);
+        auto menu = action->menu();
+        if (menu) {
+            menu->deleteLater();
+        }
+        action->deleteLater();
+    }
+    showTargetMenuActions.clear();
+    for (auto &target : targets) {
+        QString name;
+        if (target.name.isEmpty()) {
+            name = tr("%1 (used here)").arg(RAddressString(target.offset));
+        } else {
+            name = tr("%1 (%2)").arg(target.name, RAddressString(target.offset));
+        }
+        auto action = new QAction(name, this);
+        showTargetMenuActions.append(action);
+        auto menu = mainWindow->createShowInMenu(this, target.offset);
+        action->setMenu(menu);
+        QAction *copyAddress = new QAction(tr("Copy address"), menu);
+        RVA offset = target.offset;
+        connect(copyAddress, &QAction::triggered, copyAddress, [offset]() {
+            QClipboard *clipboard = QApplication::clipboard();
+            clipboard->setText(RAddressString(offset));
+        });
+        menu->addSeparator();
+        menu->addAction(copyAddress);
+    }
+    insertActions(copySeparator, showTargetMenuActions);
+}
+
 void DisassemblyContextMenu::setOffset(RVA offset)
 {
     this->offset = offset;
+
+    this->actionSetFunctionVarTypes.setVisible(true);
 }
 
 void DisassemblyContextMenu::setCanCopy(bool enabled)
@@ -235,6 +390,11 @@ void DisassemblyContextMenu::setCanCopy(bool enabled)
     this->canCopy = enabled;
 }
 
+void DisassemblyContextMenu::setCurHighlightedWord(const QString &text)
+{
+    this->curHighlightedWord = text;
+}
+
 void DisassemblyContextMenu::aboutToShowSlot()
 {
     // check if set immediate base menu makes sense
@@ -245,6 +405,48 @@ void DisassemblyContextMenu::aboutToShowSlot()
     setBaseMenu->menuAction()->setVisible(immBase);
     setBitsMenu->menuAction()->setVisible(true);
 
+    // Create structure offset menu if it makes sense
+    QString memBaseReg; // Base register
+    QVariant memDisp; // Displacement
+    if (instObject.contains("opex") && instObject["opex"].toObject().contains("operands")) {
+        // Loop through both the operands of the instruction
+        for (const QJsonValue value: instObject["opex"].toObject()["operands"].toArray()) {
+            QJsonObject operand = value.toObject();
+            if (operand.contains("type") && operand["type"].toString() == "mem" &&
+                    operand.contains("base") && !operand["base"].toString().contains("bp") &&
+                    operand.contains("disp") && operand["disp"].toVariant().toLongLong() > 0) {
+
+                    // The current operand is the one which has an immediate displacement
+                    memBaseReg = operand["base"].toString();
+                    memDisp = operand["disp"].toVariant();
+                    break;
+
+            }
+        }
+    }
+    if (memBaseReg.isEmpty()) {
+        // hide structure offset menu
+        structureOffsetMenu->menuAction()->setVisible(false);
+    } else {
+        // show structure offset menu
+        structureOffsetMenu->menuAction()->setVisible(true);
+        structureOffsetMenu->clear();
+
+        // Get the possible offsets using the "ahts" command
+        // TODO: add ahtj command to radare2 and then use it here
+        QStringList ret = Core()->cmdList("ahts " + memDisp.toString());
+        for (const QString &val : ret) {
+            if (val.isEmpty()) {
+                continue;
+            }
+            structureOffsetMenu->addAction("[" + memBaseReg + " + " + val + "]")->setData(val);
+        }
+        if (structureOffsetMenu->isEmpty()) {
+            // No possible offset was found so hide the menu
+            structureOffsetMenu->menuAction()->setVisible(false);
+        }
+    }
+
     actionAnalyzeFunction.setVisible(true);
 
     QString comment = Core()->cmd("CC." + RAddressString(offset));
@@ -261,8 +463,8 @@ void DisassemblyContextMenu::aboutToShowSlot()
 
 
     RCore *core = Core()->core();
-    RAnalFunction *fcn = r_anal_get_fcn_at (core->anal, offset, R_ANAL_FCN_TYPE_NULL);
-    RAnalFunction *in_fcn = Core()->functionAt(offset);
+    RAnalFunction *fcn = Core()->functionAt(offset);
+    RAnalFunction *in_fcn = Core()->functionIn(offset);
     RFlagItem *f = r_flag_get_i (core->flags, offset);
 
     actionDeleteFlag.setVisible(f ? true : false);
@@ -281,7 +483,8 @@ void DisassemblyContextMenu::aboutToShowSlot()
 
     // Only show retype for local vars if in a function
     if (in_fcn) {
-        actionSetFunctionVarTypes.setVisible(true);
+        auto vars = Core()->getVariables(offset);
+        actionSetFunctionVarTypes.setVisible(!vars.empty());
         actionEditFunction.setVisible(true);
         actionEditFunction.setText(tr("Edit function \"%1\"").arg(in_fcn->name));
     } else {
@@ -291,33 +494,45 @@ void DisassemblyContextMenu::aboutToShowSlot()
 
 
     // Only show "rename X used here" if there is something to rename
-    QJsonArray thingUsedHereArray = Core()->cmdj("anj @ " + QString::number(offset)).array();
-    if (!thingUsedHereArray.isEmpty()) {
+    auto thingsUsedHere = getThingUsedHere(offset);
+    if (!thingsUsedHere.isEmpty()) {
         actionRenameUsedHere.setVisible(true);
-        QJsonObject thingUsedHere = thingUsedHereArray.first().toObject();
-        if (thingUsedHere["type"] == "address") {
-            RVA offset = thingUsedHere["offset"].toVariant().toULongLong();
+        auto &thingUsedHere = thingsUsedHere.first();
+        if (thingUsedHere.type == ThingUsedHere::Type::Address) {
+            RVA offset = thingUsedHere.offset;
             actionRenameUsedHere.setText(tr("Add flag at %1 (used here)").arg(RAddressString(offset)));
-        } else {
-            if (thingUsedHere["type"] == "function") {
-                actionRenameUsedHere.setText(tr("Rename \"%1\"").arg(thingUsedHere["name"].toString()));
-            }
-            else {
-                actionRenameUsedHere.setText(tr("Rename \"%1\" (used here)").arg(thingUsedHere["name"].toString()));
-            }
+        } else if (thingUsedHere.type == ThingUsedHere::Type::Function) {
+            actionRenameUsedHere.setText(tr("Rename \"%1\"").arg(thingUsedHere.name));
+        }  else {
+            actionRenameUsedHere.setText(tr("Rename \"%1\" (used here)").arg(thingUsedHere.name));
         }
     } else {
         actionRenameUsedHere.setVisible(false);
     }
+    updateTargetMenuActions(thingsUsedHere);
 
     // Decide to show Reverse jmp option
     showReverseJmpQuery();
 
+    if (showInSubmenu.menu() != nullptr) {
+        showInSubmenu.menu()->deleteLater();
+    }
+    showInSubmenu.setMenu(mainWindow->createShowInMenu(this, offset));
+
     // Only show debug options if we are currently debugging
     debugMenu->menuAction()->setVisible(Core()->currentlyDebugging);
-    QString progCounterName = Core()->getRegisterName("PC");
+    bool hasBreakpoint = Core()->breakpointIndexAt(offset) > -1;
+    actionAddBreakpoint.setText(hasBreakpoint ?
+                                     tr("Remove breakpoint") : tr("Add breakpoint"));
+    actionAdvancedBreakpoint.setText(hasBreakpoint ?
+                                     tr("Edit breakpoint") : tr("Advanced breakpoint"));
+    QString progCounterName = Core()->getRegisterName("PC").toUpper();
     actionSetPC.setText("Set " + progCounterName + " here");
 
+    pluginActionMenuAction->setVisible(!pluginMenu->isEmpty());
+    for (QAction *pluginAction : pluginMenu->actions()) {
+        pluginAction->setData(QVariant::fromValue(offset));
+    }
 }
 
 QKeySequence DisassemblyContextMenu::getCopySequence() const
@@ -330,6 +545,11 @@ QKeySequence DisassemblyContextMenu::getCommentSequence() const
     return {Qt::Key_Semicolon};
 }
 
+QKeySequence DisassemblyContextMenu::getCopyAddressSequence() const
+{
+    return {Qt::CTRL + Qt::SHIFT + Qt::Key_C};
+}
+
 QKeySequence DisassemblyContextMenu::getSetToCodeSequence() const
 {
     return {Qt::Key_C};
@@ -381,23 +601,44 @@ QKeySequence DisassemblyContextMenu::getDisplayOptionsSequence() const
     return {}; //TODO insert correct sequence
 }
 
+QKeySequence DisassemblyContextMenu::getLinkTypeSequence() const
+{
+    return {Qt::Key_L};
+}
+
 QList<QKeySequence> DisassemblyContextMenu::getAddBPSequence() const
 {
     return {Qt::Key_F2, Qt::CTRL + Qt::Key_B};
 }
 
+QKeySequence DisassemblyContextMenu::getDefineNewFunctionSequence() const
+{
+    return {Qt::Key_P};
+}
+
+QKeySequence DisassemblyContextMenu::getEditFunctionSequence() const
+{
+    return {Qt::SHIFT + Qt::Key_P};
+}
+
+QKeySequence DisassemblyContextMenu::getUndefineFunctionSequence() const
+{
+    return {Qt::Key_U};
+}
+
+
 void DisassemblyContextMenu::on_actionEditInstruction_triggered()
 {
-    EditInstructionDialog *e = new EditInstructionDialog(this, EDIT_TEXT);
-    e->setWindowTitle(tr("Edit Instruction at %1").arg(RAddressString(offset)));
+    EditInstructionDialog e(EDIT_TEXT, this);
+    e.setWindowTitle(tr("Edit Instruction at %1").arg(RAddressString(offset)));
 
     QString oldInstructionOpcode = Core()->getInstructionOpcode(offset);
     QString oldInstructionBytes = Core()->getInstructionBytes(offset);
 
-    e->setInstruction(oldInstructionOpcode);
+    e.setInstruction(oldInstructionOpcode);
 
-    if (e->exec()) {
-        QString userInstructionOpcode = e->getInstruction();
+    if (e.exec()) {
+        QString userInstructionOpcode = e.getInstruction();
         if (userInstructionOpcode != oldInstructionOpcode) {
             Core()->editInstruction(offset, userInstructionOpcode);
 
@@ -459,14 +700,14 @@ void DisassemblyContextMenu::on_actionJmpReverse_triggered()
 
 void DisassemblyContextMenu::on_actionEditBytes_triggered()
 {
-    EditInstructionDialog *e = new EditInstructionDialog(this, EDIT_BYTES);
-    e->setWindowTitle(tr("Edit Bytes at %1").arg(RAddressString(offset)));
+    EditInstructionDialog e(EDIT_BYTES, this);
+    e.setWindowTitle(tr("Edit Bytes at %1").arg(RAddressString(offset)));
 
     QString oldBytes = Core()->getInstructionBytes(offset);
-    e->setInstruction(oldBytes);
+    e.setInstruction(oldBytes);
 
-    if (e->exec()) {
-        QString bytes = e->getInstruction();
+    if (e.exec()) {
+        QString bytes = e.getInstruction();
         if (bytes != oldBytes) {
             Core()->editBytes(offset, bytes);
 
@@ -486,7 +727,7 @@ bool DisassemblyContextMenu::writeFailed()
     msgBox.setIcon(QMessageBox::Icon::Critical);
     msgBox.setWindowTitle(tr("Write error"));
     msgBox.setText(
-        tr("Unable to complete write operation. Consider opening in write mode. \n\nWARNING: In write mode any changes will be commited to disk"));
+        tr("Unable to complete write operation. Consider opening in write mode. \n\nWARNING: In write mode any changes will be committed to disk"));
     msgBox.addButton(tr("OK"), QMessageBox::NoRole);
     QAbstractButton *reopenButton = msgBox.addButton(tr("Reopen in write mode and try again"),
                                                      QMessageBox::YesRole);
@@ -517,6 +758,16 @@ void DisassemblyContextMenu::on_actionAddBreakpoint_triggered()
     Core()->toggleBreakpoint(offset);
 }
 
+void DisassemblyContextMenu::on_actionAdvancedBreakpoint_triggered()
+{
+    int index = Core()->breakpointIndexAt(offset);
+    if (index >= 0) {
+        BreakpointsDialog::editBreakpoint(Core()->getBreakpointAt(offset), this);
+    } else {
+        BreakpointsDialog::createNewBreakpoint(offset, this);
+    }
+}
+
 void DisassemblyContextMenu::on_actionContinueUntil_triggered()
 {
     Core()->continueUntilDebug(RAddressString(offset));
@@ -525,72 +776,53 @@ void DisassemblyContextMenu::on_actionContinueUntil_triggered()
 void DisassemblyContextMenu::on_actionSetPC_triggered()
 {
     QString progCounterName = Core()->getRegisterName("PC");
-    Core()->setRegister(progCounterName, RAddressString(offset));
+    Core()->setRegister(progCounterName, RAddressString(offset).toUpper());
 }
 
 void DisassemblyContextMenu::on_actionAddComment_triggered()
 {
-    QString oldComment = Core()->cmd("CC." + RAddressString(offset));
-    // Remove newline at the end added by cmd
-    oldComment.remove(oldComment.length() - 1, 1);
-    CommentsDialog *c = new CommentsDialog(this);
-
-    if (oldComment.isNull() || oldComment.isEmpty()) {
-        c->setWindowTitle(tr("Add Comment at %1").arg(RAddressString(offset)));
-    } else {
-        c->setWindowTitle(tr("Edit Comment at %1").arg(RAddressString(offset)));
-    }
-
-    c->setComment(oldComment);
-    if (c->exec()) {
-        QString comment = c->getComment();
-        if (comment.isEmpty()) {
-            Core()->delComment(offset);
-        } else {
-            Core()->setComment(offset, comment);
-        }
-    }
+    CommentsDialog::addOrEditComment(offset, this);
 }
 
 void DisassemblyContextMenu::on_actionAnalyzeFunction_triggered()
 {
-    RenameDialog *dialog = new RenameDialog(this);
-    dialog->setWindowTitle(tr("Analyze function at %1").arg(RAddressString(offset)));
-    dialog->setPlaceholderText(tr("Function name"));
-    if (dialog->exec()) {
-        QString function_name = dialog->getName();
+    RenameDialog dialog(mainWindow);
+    dialog.setWindowTitle(tr("Analyze function at %1").arg(RAddressString(offset)));
+    dialog.setPlaceholderText(tr("Function name"));
+    if (dialog.exec()) {
+        QString function_name = dialog.getName();
         Core()->createFunctionAt(offset, function_name);
     }
 }
 
 void DisassemblyContextMenu::on_actionAddFlag_triggered()
 {
-    FlagDialog *dialog = new FlagDialog(offset, this->parentWidget());
-    dialog->exec();
+    FlagDialog dialog(offset, this->parentWidget());
+    dialog.exec();
 }
 
 void DisassemblyContextMenu::on_actionRename_triggered()
 {
     RCore *core = Core()->core();
 
-    RenameDialog *dialog = new RenameDialog(this);
+    RenameDialog dialog(mainWindow);
 
-    RAnalFunction *fcn = r_anal_get_fcn_at (core->anal, offset, R_ANAL_FCN_TYPE_NULL);
+    RAnalFunction *fcn = Core()->functionIn (offset);
     RFlagItem *f = r_flag_get_i (core->flags, offset);
     if (fcn) {
         /* Rename function */
-        dialog->setWindowTitle(tr("Rename function %1").arg(fcn->name));
-        dialog->setName(fcn->name);
-        if (dialog->exec()) {
-            QString new_name = dialog->getName();
+        dialog.setWindowTitle(tr("Rename function %1").arg(fcn->name));
+        dialog.setName(fcn->name);
+        if (dialog.exec()) {
+            QString new_name = dialog.getName();
             Core()->renameFunction(fcn->name, new_name);
         }
     } else if (f) {
         /* Rename current flag */
-        dialog->setWindowTitle(tr("Rename flag %1").arg(f->name));
-        dialog->setName(f->name);
-        if (dialog->exec()) {
-            QString new_name = dialog->getName();
+        dialog.setWindowTitle(tr("Rename flag %1").arg(f->name));
+        dialog.setName(f->name);
+        if (dialog.exec()) {
+            QString new_name = dialog.getName();
             Core()->renameFlag(f->name, new_name);
         }
     } else {
@@ -600,38 +832,39 @@ void DisassemblyContextMenu::on_actionRename_triggered()
 
 void DisassemblyContextMenu::on_actionRenameUsedHere_triggered()
 {
-    QJsonArray array = Core()->cmdj("anj @ " + QString::number(offset)).array();
+    auto array = getThingUsedHere(offset);
     if (array.isEmpty()) {
         return;
     }
 
-    QJsonObject thingUsedHere = array.first().toObject();
-    QString type = thingUsedHere.value("type").toString();
+    auto thingUsedHere = array.first();
 
-    RenameDialog *dialog = new RenameDialog(this);
+    RenameDialog dialog(mainWindow);
 
     QString oldName;
+    auto type = thingUsedHere.type;
 
-    if (type == "address") {
-        RVA offset = thingUsedHere["offset"].toVariant().toULongLong();
-        dialog->setWindowTitle(tr("Add flag at %1").arg(RAddressString(offset)));
-        dialog->setName("label." + QString::number(offset, 16));
+    if (type == ThingUsedHere::Type::Address) {
+        RVA offset = thingUsedHere.offset;
+        dialog.setWindowTitle(tr("Add flag at %1").arg(RAddressString(offset)));
+        dialog.setName("label." + QString::number(offset, 16));
     } else {
-        oldName = thingUsedHere.value("name").toString();
-        dialog->setWindowTitle(tr("Rename %1").arg(oldName));
-        dialog->setName(oldName);
+        oldName = thingUsedHere.name;
+        dialog.setWindowTitle(tr("Rename %1").arg(oldName));
+        dialog.setName(oldName);
     }
 
-    if (dialog->exec()) {
-        QString newName = dialog->getName().trimmed();
+
+    if (dialog.exec()) {
+        QString newName = dialog.getName().trimmed();
         if (!newName.isEmpty()) {
             Core()->cmd("an " + newName + " @ " + QString::number(offset));
 
-            if (type == "address" || type == "flag") {
+            if (type == ThingUsedHere::Type::Address || type == ThingUsedHere::Type::Flag) {
                 Core()->triggerFlagsChanged();
-            } else if (type == "var") {
+            } else if (type == ThingUsedHere::Type::Var) {
                 Core()->triggerVarsChanged();
-            } else if (type == "function") {
+            } else if (type == ThingUsedHere::Type::Function) {
                 Core()->triggerFunctionRenamed(oldName, newName);
             }
         }
@@ -640,29 +873,33 @@ void DisassemblyContextMenu::on_actionRenameUsedHere_triggered()
 
 void DisassemblyContextMenu::on_actionSetFunctionVarTypes_triggered()
 {
-    RAnalFunction *fcn = Core()->functionAt(offset);
+    RAnalFunction *fcn = Core()->functionIn(offset);
 
     if (!fcn) {
-        QMessageBox::critical(this, tr("Re-type function local vars"), tr("You must be in a function to define variable types."));
+        QMessageBox::critical(this, tr("Re-type function local vars"),
+                              tr("You must be in a function to define variable types."));
         return;
     }
 
-    EditVariablesDialog dialog(Core()->getOffset(), this);
+    EditVariablesDialog dialog(Core()->getOffset(), curHighlightedWord, this);
+    if (dialog.empty()) { // don't show the dialog if there are no variables
+        return;
+    }
     dialog.exec();
 }
 
 void DisassemblyContextMenu::on_actionXRefs_triggered()
 {
-    XrefsDialog *dialog = new XrefsDialog(this);
-    dialog->fillRefsForAddress(offset, RAddressString(offset), false);
-    dialog->exec();
+    XrefsDialog dialog(mainWindow, nullptr);
+    dialog.fillRefsForAddress(offset, RAddressString(offset), false);
+    dialog.exec();
 }
 
 void DisassemblyContextMenu::on_actionDisplayOptions_triggered()
 {
-    auto *dialog = new PreferencesDialog(this->window());
-    dialog->showSection(PreferencesDialog::Section::Disassembly);
-    dialog->show();
+    PreferencesDialog dialog(this->window());
+    dialog.showSection(PreferencesDialog::Section::Disassembly);
+    dialog.exec();
 }
 
 void DisassemblyContextMenu::on_actionSetToCode_triggered()
@@ -675,6 +912,47 @@ void DisassemblyContextMenu::on_actionSetAsString_triggered()
     Core()->setAsString(offset);
 }
 
+void DisassemblyContextMenu::on_actionSetAsStringRemove_triggered()
+{
+    Core()->removeString(offset);
+}
+
+void DisassemblyContextMenu::on_actionSetAsStringAdvanced_triggered()
+{
+    EditStringDialog dialog(parentWidget());
+    const int predictedStrSize = Core()->getString(offset).size();
+    dialog.setStringSizeValue(predictedStrSize);
+    dialog.setStringStartAddress(offset);
+
+    if(!dialog.exec())
+    {
+        return;
+    }
+
+    uint64_t strAddr = 0U;
+    if( !dialog.getStringStartAddress(strAddr) ) {
+        QMessageBox::critical(this->window(), tr("Wrong address"), tr("Can't edit string at this address"));
+        return;
+    }
+    CutterCore::StringTypeFormats coreStringType = CutterCore::StringTypeFormats::None;
+
+    const auto strSize = dialog.getStringSizeValue();
+    const auto strType = dialog.getStringType();
+    switch(strType)
+    {
+    case EditStringDialog::StringType::Auto:
+        coreStringType = CutterCore::StringTypeFormats::None;
+        break;
+    case EditStringDialog::StringType::ASCII_LATIN1:
+        coreStringType = CutterCore::StringTypeFormats::ASCII_LATIN1;
+        break;
+    case EditStringDialog::StringType::UTF8:
+        coreStringType = CutterCore::StringTypeFormats::UTF8;
+        break;
+    };
+
+    Core()->setAsString(strAddr, strSize, coreStringType);
+}
 
 void DisassemblyContextMenu::on_actionSetToData_triggered()
 {
@@ -692,11 +970,25 @@ void DisassemblyContextMenu::on_actionSetToData_triggered()
 
 void DisassemblyContextMenu::on_actionSetToDataEx_triggered()
 {
-    auto dialog = new SetToDataDialog(offset, this->window());
-    if (!dialog->exec()) {
+    SetToDataDialog dialog(offset, this->window());
+    if (!dialog.exec()) {
         return;
     }
-    setToData(dialog->getItemSize(), dialog->getItemCount());
+    setToData(dialog.getItemSize(), dialog.getItemCount());
+}
+
+void DisassemblyContextMenu::on_actionStructureOffsetMenu_triggered(QAction *action)
+{
+    Core()->applyStructureOffset(action->data().toString(), offset);
+}
+
+void DisassemblyContextMenu::on_actionLinkType_triggered()
+{
+    LinkTypeDialog dialog(mainWindow);
+    if (!dialog.setDefaultAddress(curHighlightedWord)) {
+        dialog.setDefaultAddress(RAddressString(offset));
+    }
+    dialog.exec();
 }
 
 void DisassemblyContextMenu::on_actionDeleteComment_triggered()
@@ -717,39 +1009,34 @@ void DisassemblyContextMenu::on_actionDeleteFunction_triggered()
 void DisassemblyContextMenu::on_actionEditFunction_triggered()
 {
     RCore *core = Core()->core();
-    EditFunctionDialog *dialog = new EditFunctionDialog(this);
+    EditFunctionDialog dialog(mainWindow);
     RAnalFunction *fcn = r_anal_get_fcn_in(core->anal, offset, 0);
 
     if (fcn) {
-        dialog->setWindowTitle(tr("Edit function %1").arg(fcn->name));
-        dialog->setNameText(fcn->name);
+        dialog.setWindowTitle(tr("Edit function %1").arg(fcn->name));
+        dialog.setNameText(fcn->name);
 
         QString startAddrText = "0x" + QString::number(fcn->addr, 16);
-        dialog->setStartAddrText(startAddrText);
-
-        QString endAddrText = "0x" + QString::number(fcn->addr + fcn->_size, 16);
-        dialog->setEndAddrText(endAddrText);
+        dialog.setStartAddrText(startAddrText);
 
         QString stackSizeText;
         stackSizeText.sprintf("%d", fcn->stack);
-        dialog->setStackSizeText(stackSizeText);
+        dialog.setStackSizeText(stackSizeText);
 
         QStringList callConList = Core()->cmd("afcl").split("\n");
         callConList.removeLast();
-        dialog->setCallConList(callConList);
-        dialog->setCallConSelected(fcn->cc);
+        dialog.setCallConList(callConList);
+        dialog.setCallConSelected(fcn->cc);
 
 
-        if (dialog->exec()) {
-            QString new_name = dialog->getNameText();
+        if (dialog.exec()) {
+            QString new_name = dialog.getNameText();
             Core()->renameFunction(fcn->name, new_name);
-            QString new_start_addr = dialog->getStartAddrText();
+            QString new_start_addr = dialog.getStartAddrText();
             fcn->addr = Core()->math(new_start_addr);
-            QString new_end_addr = dialog->getEndAddrText();
-            Core()->cmd("afu " + new_end_addr);
-            QString new_stack_size = dialog->getStackSizeText();
+            QString new_stack_size = dialog.getStackSizeText();
             fcn->stack = int(Core()->math(new_stack_size));
-            Core()->cmd("afc " + dialog->getCallConSelected());
+            Core()->cmd("afc " + dialog.getCallConSelected());
             emit Core()->functionsChanged();
         }
     }
@@ -773,7 +1060,7 @@ void DisassemblyContextMenu::setToData(int size, int repeat)
 QAction *DisassemblyContextMenu::addAnonymousAction(QString name, const char *slot,
                                                     QKeySequence keySequence)
 {
-    auto action = new QAction();
+    auto action = new QAction(this);
     addAction(action);
     anonymousActions.append(action);
     initAction(action, name, slot, keySequence);
@@ -783,6 +1070,7 @@ QAction *DisassemblyContextMenu::addAnonymousAction(QString name, const char *sl
 void DisassemblyContextMenu::initAction(QAction *action, QString name, const char *slot)
 {
     action->setParent(this);
+    parentWidget()->addAction(action);
     action->setText(name);
     if (slot) {
         connect(action, SIGNAL(triggered(bool)), this, slot);
@@ -797,10 +1085,7 @@ void DisassemblyContextMenu::initAction(QAction *action, QString name,
         return;
     }
     action->setShortcut(keySequence);
-    auto pWidget = parentWidget();
-    auto shortcut = new QShortcut(keySequence, pWidget);
-    shortcut->setContext(Qt::WidgetWithChildrenShortcut);
-    connect(shortcut, SIGNAL(activated()), this, slot);
+    action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
 }
 
 void DisassemblyContextMenu::initAction(QAction *action, QString name,
@@ -811,10 +1096,5 @@ void DisassemblyContextMenu::initAction(QAction *action, QString name,
         return;
     }
     action->setShortcuts(keySequenceList);
-    auto pWidget = parentWidget();
-    for (auto keySequence : keySequenceList) {
-        auto shortcut = new QShortcut(keySequence, pWidget);
-        shortcut->setContext(Qt::WidgetWithChildrenShortcut);
-        connect(shortcut, SIGNAL(activated()), this, slot);
-    }
+    action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
 }
