@@ -5,7 +5,7 @@
 #include <QStandardItem>
 #include <QTableView>
 
-#include "Cutter.h"
+#include "core/Cutter.h"
 #include "CutterDockWidget.h"
 
 class MainWindow;
@@ -30,6 +30,6 @@ private slots:
 private:
     std::unique_ptr<Ui::BacktraceWidget> ui;
     QStandardItemModel *modelBacktrace = new QStandardItemModel(1, 5, this);
-    QTableView *viewBacktrace = new QTableView;
+    QTableView *viewBacktrace = new QTableView(this);
     RefreshDeferrer *refreshDeferrer;
 };
\ No newline at end of file
