@@ -4,6 +4,11 @@
 #include <memory>
 #include "CutterDockWidget.h"
 
+QT_BEGIN_NAMESPACE
+QT_FORWARD_DECLARE_CLASS(QLineEdit)
+QT_FORWARD_DECLARE_CLASS(QJsonObject)
+QT_END_NAMESPACE
+
 class MainWindow;
 
 namespace Ui {
@@ -25,6 +30,8 @@ private slots:
 
 private:
     std::unique_ptr<Ui::Dashboard>   ui;
+    void setPlainText(QLineEdit *textBox, const QString &text);
+    void setBool(QLineEdit *textBox, const QJsonObject &jsonObject, const QString &key);
 };
 
 #endif // DASHBOARD_H
