@@ -1,17 +1,20 @@
 #include "DisassemblyWidget.h"
 #include "menus/DisassemblyContextMenu.h"
-#include "common/HexAsciiHighlighter.h"
-#include "common/HexHighlighter.h"
 #include "common/Configuration.h"
 #include "common/Helpers.h"
 #include "common/TempConfig.h"
+#include "common/SelectionHighlight.h"
+#include "core/MainWindow.h"
 
+#include <QApplication>
 #include <QScrollBar>
 #include <QJsonArray>
 #include <QJsonObject>
 #include <QVBoxLayout>
 #include <QRegularExpression>
 #include <QTextBlockUserData>
+#include <QPainter>
+#include <QSplitter>
 
 
 class DisassemblyTextBlockUserData: public QTextBlockUserData
@@ -36,28 +39,70 @@ static DisassemblyTextBlockUserData *getUserData(const QTextBlock &block)
 }
 
 DisassemblyWidget::DisassemblyWidget(MainWindow *main, QAction *action)
-    :   CutterDockWidget(main, action)
-    ,   mCtxMenu(new DisassemblyContextMenu(this))
+    :   MemoryDockWidget(MemoryWidgetType::Disassembly, main, action)
+    ,   mCtxMenu(new DisassemblyContextMenu(this, main))
     ,   mDisasScrollArea(new DisassemblyScrollArea(this))
     ,   mDisasTextEdit(new DisassemblyTextEdit(this))
-    ,   seekable(new CutterSeekable(this))
 {
+    setObjectName(main
+                  ? main->getUniqueObjectName(getWidgetType())
+                  : getWidgetType());
+
     topOffset = bottomOffset = RVA_INVALID;
     cursorLineOffset = 0;
+    cursorCharOffset = 0;
     seekFromCursor = false;
 
-    setWindowTitle(tr("Disassembly"));
+    setWindowTitle(getWindowTitle());
 
-    QVBoxLayout *layout = new QVBoxLayout();
+    // Instantiate the window layout
+    auto *splitter = new QSplitter;
+
+    // Setup the left frame that contains breakpoints and jumps
+    leftPanel = new DisassemblyLeftPanel(this);
+    splitter->addWidget(leftPanel);
+
+    // Setup the disassembly content
+    auto *layout = new QHBoxLayout;
     layout->addWidget(mDisasTextEdit);
     layout->setMargin(0);
     mDisasScrollArea->viewport()->setLayout(layout);
-    mDisasScrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    splitter->addWidget(mDisasScrollArea);
+    mDisasScrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
+    // Use stylesheet instead of QWidget::setFrameShape(QFrame::NoShape) to avoid
+    // issues with dark and light interface themes
+    mDisasScrollArea->setStyleSheet("QAbstractScrollArea { border: 0px transparent black; }");
+    mDisasTextEdit->setStyleSheet("QPlainTextEdit { border: 0px transparent black; }");
+    mDisasTextEdit->setFocusProxy(this);
+    mDisasTextEdit->setFocusPolicy(Qt::ClickFocus);
+    mDisasScrollArea->setFocusProxy(this);
+    mDisasScrollArea->setFocusPolicy(Qt::ClickFocus);
 
-    setWidget(mDisasScrollArea);
+    setFocusPolicy(Qt::ClickFocus);
+
+    // Behave like all widgets: highlight on focus and hover
+    connect(qApp, &QApplication::focusChanged, this, [this](QWidget* , QWidget* now) {
+        QColor borderColor = this == now
+                             ? palette().color(QPalette::Highlight)
+                             : palette().color(QPalette::WindowText).darker();
+        widget()->setStyleSheet(QString("QSplitter { border: %1px solid %2 } \n"
+                                        "QSplitter:hover { border: %1px solid %3 } \n")
+                                .arg(devicePixelRatio())
+                                .arg(borderColor.name())
+                                .arg(palette().color(QPalette::Highlight).name()));
+    });
+
+    splitter->setFrameShape(QFrame::Box);
+    // Set current widget to the splitted layout we just created
+    setWidget(splitter);
+
+    // Resize properly
+    QList<int> sizes;
+    sizes.append(3);
+    sizes.append(1);
+    splitter->setSizes(sizes);
 
     setAllowedAreas(Qt::AllDockWidgetAreas);
-    setObjectName("DisassemblyWidget");
 
     setupFonts();
     setupColors();
@@ -89,15 +134,6 @@ DisassemblyWidget::DisassemblyWidget(MainWindow *main, QAction *action)
             this, SLOT(showDisasContextMenu(const QPoint &)));
 
 
-    // Space to switch to graph
-    QShortcut *graphShortcut = new QShortcut(QKeySequence(Qt::Key_Space), this);
-    graphShortcut->setContext(Qt::WidgetWithChildrenShortcut);
-    connect(graphShortcut, &QShortcut::activated, this, [] {
-        Core()->setMemoryWidgetPriority(CutterCore::MemoryWidgetType::Graph);
-        Core()->triggerRaisePrioritizedMemoryWidget();
-    });
-
-
     connect(mDisasScrollArea, SIGNAL(scrollLines(int)), this, SLOT(scrollInstructions(int)));
     connect(mDisasScrollArea, SIGNAL(disassemblyResized()), this, SLOT(updateMaxLines()));
 
@@ -108,8 +144,6 @@ DisassemblyWidget::DisassemblyWidget(MainWindow *main, QAction *action)
         }
     });
 
-    connect(Core(), SIGNAL(raisePrioritizedMemoryWidget(CutterCore::MemoryWidgetType)), this,
-            SLOT(raisePrioritizedMemoryWidget(CutterCore::MemoryWidgetType)));
     connect(Core(), SIGNAL(commentsChanged()), this, SLOT(refreshDisasm()));
     connect(Core(), SIGNAL(flagsChanged()), this, SLOT(refreshDisasm()));
     connect(Core(), SIGNAL(functionsChanged()), this, SLOT(refreshDisasm()));
@@ -127,67 +161,71 @@ DisassemblyWidget::DisassemblyWidget(MainWindow *main, QAction *action)
     connect(Config(), SIGNAL(fontsUpdated()), this, SLOT(fontsUpdatedSlot()));
     connect(Config(), SIGNAL(colorsUpdated()), this, SLOT(colorsUpdatedSlot()));
 
-    connect(this, &QDockWidget::visibilityChanged, this, [](bool visibility) {
-        bool emptyGraph = (Core()->getMemoryWidgetPriority() == CutterCore::MemoryWidgetType::Graph
-                           && Core()->isGraphEmpty());
-        if (visibility && !emptyGraph) {
-            Core()->setMemoryWidgetPriority(CutterCore::MemoryWidgetType::Disassembly);
-        }
-    });
-
     connect(Core(), &CutterCore::refreshAll, this, [this]() {
         refreshDisasm(seekable->getOffset());
     });
+    refreshDisasm(seekable->getOffset());
 
     connect(mCtxMenu, SIGNAL(copy()), mDisasTextEdit, SLOT(copy()));
 
-    // Dirty
-    QShortcut *shortcut_escape = new QShortcut(QKeySequence(Qt::Key_Escape), this);
-    shortcut_escape->setContext(Qt::WidgetShortcut);
-    connect(shortcut_escape, SIGNAL(activated()), this, SLOT(seekPrev()));
-
     mCtxMenu->addSeparator();
-    syncIt.setText(tr("Sync/unsync offset"));
-    mCtxMenu->addAction(&syncIt);
-    connect(&syncIt, SIGNAL(triggered(bool)), this, SLOT(toggleSync()));
+    mCtxMenu->addAction(&syncAction);
     connect(seekable, &CutterSeekable::seekableSeekChanged, this, &DisassemblyWidget::on_seekChanged);
 
-#define ADD_SHORTCUT(ksq, slot) { \
-    QShortcut *s = new QShortcut((ksq), this); \
-    s->setContext(Qt::WidgetShortcut); \
-    connect(s, &QShortcut::activated, this, (slot)); \
-}
-    ADD_SHORTCUT(QKeySequence(Qt::Key_J), [this]() {
+    addActions(mCtxMenu->actions());
+
+#define ADD_ACTION(ksq, ctx, slot) {\
+    QAction *a = new QAction(this); \
+    a->setShortcut(ksq); \
+    a->setShortcutContext(ctx); \
+    addAction(a); \
+    connect(a, &QAction::triggered, this, (slot)); }
+
+    // Space to switch to graph
+    ADD_ACTION(Qt::Key_Space, Qt::WidgetWithChildrenShortcut, [this] {
+        mainWindow->showMemoryWidget(MemoryWidgetType::Graph);
+    })
+
+    ADD_ACTION(Qt::Key_Escape, Qt::WidgetWithChildrenShortcut, &DisassemblyWidget::seekPrev)
+
+    ADD_ACTION(Qt::Key_J, Qt::WidgetWithChildrenShortcut, [this]() {
         moveCursorRelative(false, false);
     })
-    ADD_SHORTCUT(QKeySequence::MoveToNextLine, [this]() {
+    ADD_ACTION(QKeySequence::MoveToNextLine, Qt::WidgetWithChildrenShortcut, [this]() {
         moveCursorRelative(false, false);
     })
-    ADD_SHORTCUT(QKeySequence(Qt::Key_K), [this]() {
+    ADD_ACTION(Qt::Key_K, Qt::WidgetWithChildrenShortcut, [this]() {
         moveCursorRelative(true, false);
     })
-    ADD_SHORTCUT(QKeySequence::MoveToPreviousLine, [this]() {
+    ADD_ACTION(QKeySequence::MoveToPreviousLine, Qt::WidgetWithChildrenShortcut, [this]() {
         moveCursorRelative(true, false);
     })
-    ADD_SHORTCUT(QKeySequence::MoveToNextPage, [this]() {
+    ADD_ACTION(QKeySequence::MoveToNextPage, Qt::WidgetWithChildrenShortcut, [this]() {
         moveCursorRelative(false, true);
     })
-    ADD_SHORTCUT(QKeySequence::MoveToPreviousPage, [this]() {
+    ADD_ACTION(QKeySequence::MoveToPreviousPage, Qt::WidgetWithChildrenShortcut, [this]() {
         moveCursorRelative(true, true);
     })
-    ADD_SHORTCUT(QKeySequence(Qt::CTRL + Qt::Key_Plus), &DisassemblyWidget::zoomIn)
-    ADD_SHORTCUT(QKeySequence(Qt::CTRL + Qt::Key_Minus), &DisassemblyWidget::zoomOut)
-#undef ADD_SHORTCUT
+#undef ADD_ACTION
 }
 
-void DisassemblyWidget::toggleSync()
+void DisassemblyWidget::setPreviewMode(bool previewMode)
 {
-    QString windowTitle = tr("Disassembly");
-    seekable->toggleSynchronization();
-    if (seekable->isSynchronized()) {
-        setWindowTitle(windowTitle);
-    } else {
-        setWindowTitle(windowTitle + CutterSeekable::tr(" (unsynced)"));
+    mDisasTextEdit->setContextMenuPolicy(previewMode
+                                         ? Qt::NoContextMenu
+                                         : Qt::CustomContextMenu);
+    mCtxMenu->setEnabled(!previewMode);
+    for (auto action : mCtxMenu->actions()) {
+        action->setEnabled(!previewMode);
+    }
+    for (auto action : actions()) {
+        if (action->shortcut() == Qt::Key_Space ||
+            action->shortcut() == Qt::Key_Escape) {
+            action->setEnabled(!previewMode);
+        }
+    }
+    if (previewMode) {
+        seekable->setSynchronization(false);
     }
 }
 
@@ -196,6 +234,21 @@ QWidget *DisassemblyWidget::getTextWidget()
     return mDisasTextEdit;
 }
 
+QString DisassemblyWidget::getWidgetType()
+{
+    return "Disassembly";
+}
+
+QFontMetrics DisassemblyWidget::getFontMetrics()
+{
+    return mDisasTextEdit->fontMetrics();
+}
+
+QList<DisassemblyLine> DisassemblyWidget::getLines()
+{
+    return lines;
+}
+
 void DisassemblyWidget::refreshDisasm(RVA offset)
 {
     if(!disasmRefresh->attemptRefresh(offset == RVA_INVALID ? nullptr : new RVA(offset))) {
@@ -221,12 +274,12 @@ void DisassemblyWidget::refreshDisasm(RVA offset)
     int horizontalScrollValue = mDisasTextEdit->horizontalScrollBar()->value();
     mDisasTextEdit->setLockScroll(true); // avoid flicker
 
-    QList<DisassemblyLine> disassemblyLines;
+    // Retrieve disassembly lines
     {
         TempConfig tempConfig;
-        tempConfig.set("scr.html", true)
-        .set("scr.color", COLOR_MODE_16M);
-        disassemblyLines = Core()->disassembleLines(topOffset, maxLines);
+        tempConfig.set("scr.color", COLOR_MODE_16M)
+		.set("asm.lines", false);
+        lines = Core()->disassembleLines(topOffset, maxLines);
     }
 
     connectCursorPositionChanged(true);
@@ -234,7 +287,7 @@ void DisassemblyWidget::refreshDisasm(RVA offset)
     mDisasTextEdit->document()->clear();
     QTextCursor cursor(mDisasTextEdit->document());
     QTextBlockFormat regular = cursor.blockFormat();
-    for (const DisassemblyLine &line : disassemblyLines) {
+    for (const DisassemblyLine &line : lines) {
         if (line.offset < topOffset) { // overflow
             break;
         }
@@ -250,8 +303,8 @@ void DisassemblyWidget::refreshDisasm(RVA offset)
         cursor.setBlockFormat(regular);
     }
 
-    if (!disassemblyLines.isEmpty()) {
-        bottomOffset = disassemblyLines[qMin(disassemblyLines.size(), maxLines) - 1].offset;
+    if (!lines.isEmpty()) {
+        bottomOffset = lines[qMin(lines.size(), maxLines) - 1].offset;
         if (bottomOffset < topOffset) {
             bottomOffset = RVA_MAX;
         }
@@ -273,6 +326,9 @@ void DisassemblyWidget::refreshDisasm(RVA offset)
 
     mDisasTextEdit->setLockScroll(false);
     mDisasTextEdit->horizontalScrollBar()->setValue(horizontalScrollValue);
+
+    // Refresh the left panel (trigger paintEvent)
+    leftPanel->update();
 }
 
 
@@ -312,35 +368,18 @@ bool DisassemblyWidget::updateMaxLines()
     return false;
 }
 
-void DisassemblyWidget::zoomIn()
-{
-    mDisasTextEdit->zoomIn();
-    updateMaxLines();
-}
-
-void DisassemblyWidget::zoomOut()
-{
-    mDisasTextEdit->zoomOut();
-    updateMaxLines();
-}
-
 void DisassemblyWidget::highlightCurrentLine()
 {
     QList<QTextEdit::ExtraSelection> extraSelections;
 
-    QColor highlightColor = ConfigColor("highlight");
+    QColor highlightColor = ConfigColor("lineHighlight");
     QColor highlightPCColor = ConfigColor("highlightPC");
-    QColor highlightWordColor = ConfigColor("highlightWord");
 
     // Highlight the current word
     QTextCursor cursor = mDisasTextEdit->textCursor();
     cursor.select(QTextCursor::WordUnderCursor);
     QString searchString = cursor.selectedText();
-
-    cursor.movePosition(QTextCursor::StartOfLine);
-    int listStartPos = cursor.position();
-    cursor.movePosition(QTextCursor::EndOfLine);
-    int lineEndPos = cursor.position();
+    curHighlightedWord = searchString;
 
     // Highlight the current line
     QTextEdit::ExtraSelection highlightSelection;
@@ -365,26 +404,7 @@ void DisassemblyWidget::highlightCurrentLine()
     }
 
     // Highlight all the words in the document same as the current one
-    QTextDocument *document = mDisasTextEdit->document();
-
-    highlightSelection.cursor = cursor;
-    highlightSelection.cursor.movePosition(QTextCursor::Start, QTextCursor::MoveAnchor);
-
-    while (!highlightSelection.cursor.isNull() && !highlightSelection.cursor.atEnd()) {
-        highlightSelection.cursor = document->find(searchString, highlightSelection.cursor,
-                                                   QTextDocument::FindWholeWords);
-
-        if (!highlightSelection.cursor.isNull()) {
-            if (highlightSelection.cursor.position() >= listStartPos
-                    && highlightSelection.cursor.position() <= lineEndPos) {
-                highlightSelection.format.setBackground(highlightWordColor);
-            } else {
-                highlightSelection.format.setBackground(highlightWordColor);
-            }
-
-            extraSelections.append(highlightSelection);
-        }
-    }
+    extraSelections.append(createSameWordsSelections(mDisasTextEdit, curHighlightedWord));
 
     // highlight PC line
     RVA PCAddr = Core()->getProgramCounterValue();
@@ -448,7 +468,7 @@ void DisassemblyWidget::updateCursorPosition()
 
     if (offset < topOffset || (offset > bottomOffset && bottomOffset != RVA_INVALID)) {
         mDisasTextEdit->moveCursor(QTextCursor::Start);
-        mDisasTextEdit->setExtraSelections({});
+        mDisasTextEdit->setExtraSelections(createSameWordsSelections(mDisasTextEdit, curHighlightedWord));
     } else {
         RVA currentCursorOffset = readCurrentDisassemblyOffset();
         QTextCursor originalCursor = mDisasTextEdit->textCursor();
@@ -462,6 +482,10 @@ void DisassemblyWidget::updateCursorPosition()
                 if (cursorLineOffset > 0) {
                     cursor.movePosition(QTextCursor::Down, QTextCursor::MoveAnchor, cursorLineOffset);
                 }
+                if (cursorCharOffset > 0) {
+                    cursor.movePosition(QTextCursor::StartOfLine);
+                    cursor.movePosition(QTextCursor::Right, QTextCursor::MoveAnchor, cursorCharOffset);
+                }
 
                 mDisasTextEdit->setTextCursor(cursor);
                 highlightCurrentLine();
@@ -505,6 +529,7 @@ void DisassemblyWidget::cursorPositionChanged()
 
     cursorLineOffset = 0;
     QTextCursor c = mDisasTextEdit->textCursor();
+    cursorCharOffset = c.positionInBlock();
     while (c.blockNumber() > 0) {
         c.movePosition(QTextCursor::PreviousBlock);
         if (readDisassemblyOffset(c) != offset) {
@@ -518,6 +543,14 @@ void DisassemblyWidget::cursorPositionChanged()
     seekFromCursor = false;
     highlightCurrentLine();
     mCtxMenu->setCanCopy(mDisasTextEdit->textCursor().hasSelection());
+    if (mDisasTextEdit->textCursor().hasSelection()) {
+        // A word is selected so use it
+        mCtxMenu->setCurHighlightedWord(mDisasTextEdit->textCursor().selectedText());
+    } else {
+        // No word is selected so use the word under the cursor
+        mCtxMenu->setCurHighlightedWord(curHighlightedWord);
+    }
+    leftPanel->update();
 }
 
 void DisassemblyWidget::moveCursorRelative(bool up, bool page)
@@ -581,39 +614,47 @@ void DisassemblyWidget::moveCursorRelative(bool up, bool page)
     }
 }
 
+void DisassemblyWidget::jumpToOffsetUnderCursor(const QTextCursor &cursor)
+{
+    RVA offset = readDisassemblyOffset(cursor);
+    seekable->seekToReference(offset);
+}
+
 bool DisassemblyWidget::eventFilter(QObject *obj, QEvent *event)
 {
     if (event->type() == QEvent::MouseButtonDblClick
         && (obj == mDisasTextEdit || obj == mDisasTextEdit->viewport())) {
         QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
 
-        QTextCursor cursor = mDisasTextEdit->cursorForPosition(QPoint(mouseEvent->x(), mouseEvent->y()));
-        RVA offset = readDisassemblyOffset(cursor);
-
-        RVA jump = Core()->getOffsetJump(offset);
-
-        if (jump == RVA_INVALID) {
-            bool ok;
-            RVA xref = Core()->cmdj("axfj@" + QString::number(
-                offset)).array().first().toObject().value("to").toVariant().toULongLong(&ok);
-            if (ok) {
-                jump = xref;
-            }
-        }
-
-        if (jump != RVA_INVALID) {
-            seekable->seek(jump);
-        }
+        const QTextCursor& cursor = mDisasTextEdit->cursorForPosition(QPoint(mouseEvent->x(), mouseEvent->y()));
+        jumpToOffsetUnderCursor(cursor);
 
         return true;
     }
-    return CutterDockWidget::eventFilter(obj, event);
+
+    return MemoryDockWidget::eventFilter(obj, event);
+}
+
+void DisassemblyWidget::keyPressEvent(QKeyEvent *event)
+{
+    if(event->key() == Qt::Key_Return) {
+        const QTextCursor cursor = mDisasTextEdit->textCursor();
+        jumpToOffsetUnderCursor(cursor);
+    }
+
+    MemoryDockWidget::keyPressEvent(event);
+}
+
+QString DisassemblyWidget::getWindowTitle() const
+{
+    return tr("Disassembly");
 }
 
 void DisassemblyWidget::on_seekChanged(RVA offset)
 {
     if (!seekFromCursor) {
         cursorLineOffset = 0;
+        cursorCharOffset = 0;
     }
 
     if (topOffset != RVA_INVALID
@@ -627,15 +668,6 @@ void DisassemblyWidget::on_seekChanged(RVA offset)
     mCtxMenu->setOffset(offset);
 }
 
-void DisassemblyWidget::raisePrioritizedMemoryWidget(CutterCore::MemoryWidgetType type)
-{
-    bool emptyGraph = (type == CutterCore::MemoryWidgetType::Graph && Core()->isGraphEmpty());
-    if (type == CutterCore::MemoryWidgetType::Disassembly || emptyGraph) {
-        raise();
-        setFocus();
-    }
-}
-
 void DisassemblyWidget::fontsUpdatedSlot()
 {
     setupFonts();
@@ -691,6 +723,12 @@ void DisassemblyScrollArea::resetScrollBars()
     verticalScrollBar()->blockSignals(false);
 }
 
+qreal DisassemblyTextEdit::textOffset() const
+{
+    return (blockBoundingGeometry(document()->begin()).topLeft() +
+            contentOffset()).y();
+}
+
 bool DisassemblyTextEdit::viewportEvent(QEvent *event)
 {
     switch (event->type()) {
@@ -727,3 +765,163 @@ void DisassemblyWidget::seekPrev()
 {
     Core()->seekPrev();
 }
+
+/*********************
+ * Left panel
+ *********************/
+
+struct Range {
+    Range(RVA v1, RVA v2)
+        : from(v1), to(v2) { if (from > to) std::swap(from, to); }
+    RVA from;
+    RVA to;
+
+    inline bool contains(const Range& other) const
+    {
+        return from <= other.from && to >= other.to;
+    }
+
+    inline bool contains(RVA point) const
+    {
+        return from <= point && to >= point;
+    }
+};
+
+DisassemblyLeftPanel::DisassemblyLeftPanel(DisassemblyWidget *disas)
+{
+    this->disas = disas;
+}
+
+void DisassemblyLeftPanel::wheelEvent(QWheelEvent *event) {
+    int count = -(event->angleDelta() / 15).y();
+    count -= (count > 0 ? 5 : -5);
+
+    this->disas->scrollInstructions(count);
+}
+
+void DisassemblyLeftPanel::paintEvent(QPaintEvent *event)
+{
+    Q_UNUSED(event)
+
+    using namespace std;
+    constexpr int penSizePix = 1;
+    constexpr int distanceBetweenLines = 10;
+    constexpr int arrowWidth = 5;
+    int rightOffset = size().rwidth();
+    auto tEdit = qobject_cast<DisassemblyTextEdit*>(disas->getTextWidget());
+    int topOffset = int(tEdit->contentsMargins().top() + tEdit->textOffset());
+    int lineHeight = disas->getFontMetrics().height();
+    QColor arrowColorDown = ConfigColor("flow");
+    QColor arrowColorUp = ConfigColor("cflow");
+    QPainter p(this);
+    QPen penDown(arrowColorDown, penSizePix, Qt::SolidLine, Qt::FlatCap, Qt::RoundJoin);
+    QPen penUp(arrowColorUp, penSizePix, Qt::SolidLine, Qt::FlatCap, Qt::RoundJoin);
+    // Fill background
+    p.fillRect(event->rect(), Config()->getColor("gui.background").darker(115));
+
+    QList<DisassemblyLine> lines = disas->getLines();
+
+    QMap<RVA, int> linesPixPosition;
+    QMap<RVA, pair<RVA, int>> arrowInfo; /* offset -> (arrow, layer of arrow) */
+    int nLines = 0;
+    for (const auto& line : lines) {
+        linesPixPosition[line.offset] = nLines * lineHeight + lineHeight / 2 + topOffset;
+        nLines++;
+        if (line.arrow != RVA_INVALID) {
+            arrowInfo.insert(line.offset, { line.arrow, -1 });
+        }
+    }
+
+    for (auto it = arrowInfo.begin(); it != arrowInfo.end(); it++) {
+        Range currRange = { it.key(), it.value().first };
+        it.value().second = it.value().second == -1
+                            ? 1
+                            : it.value().second;
+        for (auto innerIt = arrowInfo.begin(); innerIt != arrowInfo.end(); innerIt++) {
+            if (innerIt == it) {
+                continue;
+            }
+            Range innerRange = { innerIt.key(), innerIt.value().first };
+            if (currRange.contains(innerRange) || currRange.contains(innerRange.from)) {
+                it.value().second++;
+            }
+        }
+    }
+
+    // I'm sorry this loop below, but it is only way I see how to implement the feature
+    while (true) {
+        bool correction = false;
+        bool correction2 = false;
+        for (auto it = arrowInfo.begin(); it != arrowInfo.end(); it++) {
+            int minDistance = INT32_MAX;
+            Range currRange = { it.key(), it.value().first };
+            for (auto innerIt = arrowInfo.begin(); innerIt != arrowInfo.end(); innerIt++) {
+                if (innerIt == it) {
+                    continue;
+                }
+                Range innerRange = { innerIt.key(), innerIt.value().first };
+                if (it.value().second == innerIt.value().second &&
+                    (currRange.contains(innerRange) || currRange.contains(innerRange.from))) {
+                    it.value().second++;
+                    correction = true;
+                }
+                int distance = it.value().second - innerIt.value().second;
+                if (distance > 0 && distance < minDistance) {
+                    minDistance = distance;
+                }
+            }
+            if (minDistance > 1 && minDistance != INT32_MAX) {
+                correction2 = true;
+                it.value().second -= minDistance - 1;
+            }
+        }
+        if (!correction && !correction2) {
+            break;
+        }
+    }
+
+    const RVA currOffset = disas->getSeekable()->getOffset();
+    qreal pixelRatio = qhelpers::devicePixelRatio(p.device());
+    // Draw the lines
+    for (const auto& l : lines) {
+        int lineOffset = int((distanceBetweenLines * arrowInfo[l.offset].second + distanceBetweenLines) *
+                         pixelRatio);
+        // Skip until we reach a line that jumps to a destination
+        if (l.arrow == RVA_INVALID) {
+            continue;
+        }
+
+        bool jumpDown = l.arrow > l.offset;
+        p.setPen(jumpDown ? penDown : penUp);
+        if (l.offset == currOffset || l.arrow == currOffset) {
+            QPen pen = p.pen();
+            pen.setWidthF((penSizePix * 3) / 2.0);
+            p.setPen(pen);
+        }
+        bool endVisible = true;
+
+        int currentLineYPos = linesPixPosition[l.offset];
+        int lineArrowY = linesPixPosition.value(l.arrow, -1);
+
+        if (lineArrowY == -1) {
+            lineArrowY = jumpDown
+                              ? geometry().bottom()
+                              : 0;
+            endVisible = false;
+        }
+
+        // Draw the lines
+        p.drawLine(rightOffset, currentLineYPos, rightOffset - lineOffset, currentLineYPos);
+        p.drawLine(rightOffset - lineOffset, currentLineYPos, rightOffset - lineOffset, lineArrowY);
+
+        if (endVisible) {
+            p.drawLine(rightOffset - lineOffset, lineArrowY, rightOffset, lineArrowY);
+
+            QPainterPath arrow;
+            arrow.moveTo(rightOffset - arrowWidth, lineArrowY + arrowWidth);
+            arrow.lineTo(rightOffset - arrowWidth, lineArrowY - arrowWidth);
+            arrow.lineTo(rightOffset, lineArrowY);
+            p.fillPath(arrow, p.pen().brush());
+        }
+    }
+}
