@@ -1,16 +1,17 @@
 #ifndef RESOURCESWIDGET_H
 #define RESOURCESWIDGET_H
 
-#include "Cutter.h"
+#include "core/Cutter.h"
 #include "CutterDockWidget.h"
 #include "CutterTreeView.h"
+#include "common/AddressableItemModel.h"
+#include "widgets/ListDockWidget.h"
 
-#include <QAbstractListModel>
 
 class MainWindow;
 class ResourcesWidget;
 
-class ResourcesModel : public QAbstractListModel
+class ResourcesModel : public AddressableItemModel<QAbstractListModel>
 {
     Q_OBJECT
 
@@ -29,14 +30,17 @@ public:
     QVariant data(const QModelIndex &index, int role) const override;
     QVariant headerData(int section, Qt::Orientation orientation,
                         int role = Qt::DisplayRole) const override;
+
+    RVA address(const QModelIndex &index) const override;
 };
 
-class ResourcesWidget : public CutterDockWidget
+class ResourcesWidget : public ListDockWidget
 {
     Q_OBJECT
 
 private:
     ResourcesModel *model;
+    AddressableFilterProxyModel *filterModel;
     CutterTreeView *view;
     QList<ResourcesDescription> resources;
 
@@ -45,7 +49,6 @@ public:
 
 private slots:
     void refreshResources();
-    void onDoubleClicked(const QModelIndex &);
 };
 
 #endif // RESOURCESWIDGET_H
